package lcds.examples.bookie.dao.beans;

import java.util.Collection;
import java.util.List;

import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import lcds.examples.bookie.dao.BookDAO;
import lcds.examples.bookie.entity.Book;
import lcds.examples.bookie.entity.Person;
import lcds.examples.bookie.entity.Subject;


@SuppressWarnings("unchecked")
@Stateless
@Local(value={BookDAO.class})
public class BookDAOBean implements BookDAO {

	@PersistenceContext
	private EntityManager entityManager;

	public void persist(Book transientInstance) {
		try {
			entityManager.persist(transientInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void remove(Book persistentInstance) {
		try {
			entityManager.remove(persistentInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void removeDetached(Book detachedInstance) {
		remove(merge(detachedInstance));
	}
	
	public Book merge(Book detachedInstance) {
		try {
			Book result = entityManager.merge(detachedInstance);
			return result;
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public Book findById(int id) {
		try {
			Book instance = entityManager.find(Book.class, id);
			return instance;
		} catch (RuntimeException re) {
			throw re;
		}
	}
	
	public List<Book> getAll() {
		Query q = entityManager.createQuery(
				"from Book b" +
				" order by b.title"	
			);
			List<Book> results = q.getResultList();
			return results;
	}

	public List<Book> findByAuthor(Person author) {
		Query q = entityManager.createQuery(
			"from Book b" +
			" where b.author.id = :author_id"	
		);
		q.setParameter("author_id", author.getId());
		List<Book> results = q.getResultList();
		return results;
	}
	
	public List<Book> findBySubject(Subject subject) {
		Query q = entityManager.createQuery(
			"from Book b" +
			" where b.subject.id = :subject_id"	
		);
		q.setParameter("subject_id", subject.getId());
		List<Book> results = q.getResultList();
		return results;
	}

	public Collection<Book> findByName(String title) {
		Query q = entityManager.createQuery(
				"from Book b" +
				" where b.title like '%" + title + "%'" +
				" order by b.title"	
			);
		List<Book> results = q.getResultList();
		return results;
	}
	
}
