package lcds.examples.bookie.dao.beans;

import java.util.Collection;
import java.util.List;

import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import lcds.examples.bookie.dao.PersonDAO;
import lcds.examples.bookie.entity.Person;


@SuppressWarnings("unchecked")
@Stateless
@Local(value={PersonDAO.class})
public class PersonDAOBean implements PersonDAO {

	@PersistenceContext
	private EntityManager entityManager;

	public void persist(Person transientInstance) {
		try {
			entityManager.persist(transientInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void remove(Person persistentInstance) {
		try {
			entityManager.remove(persistentInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void removeDetached(Person detachedInstance) {
		remove(merge(detachedInstance));
	}
	
	public Person merge(Person detachedInstance) {
		try {
			Person result = entityManager.merge(detachedInstance);
			return result;
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public Person findById(int id) {
		try {
			Person instance = entityManager.find(Person.class, id);
			return instance;
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public List<Person> getAll() {
		Query q = entityManager.createQuery(
			" from Person p" +
			" order by p.lastName, p.firstName"
		);
		return q.getResultList();
	}
	
	public List<Person> getAllAuthors() {
		Query q = entityManager.createQuery(
			" from Person p" +
			" where p.author = true" +
			" order by p.lastName, p.firstName"
		);
		return q.getResultList();
	}
	
	public Collection<Person> findByName(String name, boolean limitToAuthors) {
		String query = " from Person p" +
		" where (p.firstName like '%" + name + "%'" +
		" or p.lastName like '%" + name + "%')";
		if (limitToAuthors) query += " and p.author = true";
		query += " order by p.lastName, p.firstName";
		Query q = entityManager.createQuery(query);
		return q.getResultList();
	}

	public Person findByCardNumber(String cardNumber) {
		Query q = entityManager.createQuery(
			" from Person p" +
			" where p.cardNumber = :cardNumber"
		);
		q.setParameter("cardNumber", cardNumber);
		Person p;
		try {
			p = (Person)q.getSingleResult();
			return p;
		} catch (NoResultException e) {
			return null;
		}
	}
}
