package lcds.examples.bookie.dao.beans;

import java.util.List;

import javax.annotation.Resource;
import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import lcds.examples.bookie.dao.ReservationDAO;
import lcds.examples.bookie.entity.Reservation;


@SuppressWarnings("unchecked")
@Stateless
@Local(value={ReservationDAO.class})
public class ReservationDAOBean implements ReservationDAO {

	@PersistenceContext
	public EntityManager entityManager;

	@Resource(mappedName="TopicConnectionFactory")
	private ConnectionFactory connectionFactory;
	
	@Resource(mappedName="topic/outstandingReservationTopic")
	private Topic outstandingReservationTopic;

	public void persist(Reservation transientInstance) throws Exception {
		try {
			entityManager.persist(transientInstance);
			sendReservationMessage(transientInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void remove(Reservation persistentInstance) {
		try {
			entityManager.remove(persistentInstance);
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public void removeDetached(Reservation detachedInstance) throws Exception {
		remove(merge(detachedInstance));
	}
	
	public Reservation merge(Reservation detachedInstance) throws Exception {
		try {
			Boolean isNew = (detachedInstance.getId() == 0);
			Reservation result = entityManager.merge(detachedInstance);
			if (isNew) {
				sendReservationMessage(result);
			}
			return result;
		} catch (RuntimeException re) {
			throw re;
		}
	}

	public Reservation findById(int id) {
		try {
			Reservation instance = entityManager.find(Reservation.class, id);
			return instance;
		} catch (RuntimeException re) {
			throw re;
		}
	}
	
	public List<Reservation> getAllAwaitingCheckOut() {
		Query q = entityManager.createQuery(
				"from Reservation r" +
				" where reservedOn is null" +
				" order by createdOn"
			);
			List<Reservation> results = q.getResultList();
			return results;
	}
	
	public List<Reservation> getAllCheckedOut() {
		Query q = entityManager.createQuery(
				"from Reservation r" +
				" where reservedOn is not null" +
				" order by reservedTill"
			);
			List<Reservation> results = q.getResultList();
			return results;
	}
	
	private void sendReservationMessage(Reservation reservation) throws Exception {
		Connection connection = connectionFactory.createConnection();
		Session session = connection.createSession(true, 0);
		MessageProducer producer = session.createProducer(outstandingReservationTopic);
		ObjectMessage message = session.createObjectMessage();
		message.setObject(reservation);
		producer.send(message);
		connection.close();
	}
}
