package lcds.examples.bookie.entity;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.EntityListeners;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import lcds.examples.bookie.entity.listeners.CreateUpdateTimestampEventListener;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;


@MappedSuperclass
@EntityListeners({CreateUpdateTimestampEventListener.class})
public abstract class BaseEntity implements Serializable {
	
	private Integer id;
	private Calendar createdOn;
	private Calendar updatedOn;

	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	@Column(name="id")
	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="created_on")
	public Calendar getCreatedOn() {
		return createdOn;
	}

	public void setCreatedOn(Calendar createdOn) {
		this.createdOn = createdOn;
	}

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="updated_on")
	public Calendar getUpdatedOn() {
		return updatedOn;
	}

	public void setUpdatedOn(Calendar updatedOn) {
		this.updatedOn = updatedOn;
	}

	public String toString() {
		return new ToStringBuilder(this)
		.append("id", id)
		.append("createdOn", createdOn)
		.append("updatedOn", updatedOn)
		.toString();
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof BaseEntity == false) {
			return false;
		}
		
		if (this == obj) {
			return true;
		}
		BaseEntity theOther = (BaseEntity)obj;
		
		return new EqualsBuilder()
		.append(id, theOther.getId())
		.append(createdOn, theOther.getCreatedOn())
		.append(updatedOn, theOther.getUpdatedOn())
		.isEquals();
	}
	
	public int hashCode() {
		return new HashCodeBuilder(45, 89)
		.append(id)
		.append(createdOn)
		.append(updatedOn)
		.toHashCode();
	}
}
