package lcds.examples.bookie;

import java.util.Hashtable;

import javax.naming.InitialContext;

import lcds.examples.bookie.dao.DAOLookupHelper;

import org.dbunit.database.DatabaseConnection;
import org.jboss.ejb3.embedded.EJB3StandaloneBootstrap;
import org.jboss.ejb3.embedded.EJB3StandaloneDeployer;
import org.testng.annotations.AfterSuite;
import org.testng.annotations.BeforeSuite;
import org.testng.annotations.Test;


@Test(groups="functional")
public abstract class BaseEJBTest extends BaseTest {

	private static EJB3StandaloneDeployer deployer;
	private static ConnectionHelper connectionHelper;
	private static DatabaseConnection connection;

	@BeforeSuite
	@SuppressWarnings("unchecked")
	public static void startEmbeddedJBoss() throws Exception {
		EJB3StandaloneBootstrap.ignoredJars.add("rt.jar");
		EJB3StandaloneBootstrap.boot(null);

		deployer = EJB3StandaloneBootstrap.createDeployer();
		deployer.getArchivesByResource().add("META-INF/persistence.xml");
		deployer.create();
		deployer.start();
		
		// Configure the DAOLookupHelper
		DAOLookupHelper.getInstance().setJNDIPrefix("");
	}
	
	@BeforeSuite
	public static void setUpConnection() throws Exception {
		connectionHelper = new ConnectionHelper();
	}

	@AfterSuite
	public static void stopEmbeddedJBoss() {
		try {
			if (deployer != null) {
				deployer.stop();
				deployer.destroy();
			}
			EJB3StandaloneBootstrap.shutdown();
		} catch (Exception e) {
			System.out.println("Error shutting down embedded JBoss after tests: " + e.toString());
		}
	}

	@AfterSuite
	protected void tearDownDatabase() throws Exception {
		if (connection != null) {
			connection.close();
		}
	}
	
	protected static DatabaseConnection getDatabaseConnection() throws Exception {
		if (connection == null) {
			connection = new DatabaseConnection(connectionHelper.getConnection());
		}
		return connection;
	}

	protected Object lookup(String name) throws Exception {
		Hashtable<String, String> props = new Hashtable<String, String>();
		props.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
		props.put("java.naming.factory.initial", "org.jnp.interfaces.LocalOnlyContextFactory");
		InitialContext ic = new InitialContext(props);
		return ic.lookup(name);
	}
	
}
