package lcds.examples.bookie.entity;

import java.io.FileInputStream;
import java.net.URL;
import java.util.Collection;
import java.util.List;

import lcds.examples.bookie.BaseEJBTest;
import lcds.examples.bookie.dao.BookDAO;
import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.dao.PersonDAO;
import lcds.examples.bookie.dao.SubjectDAO;
import lcds.examples.bookie.entity.Book;
import lcds.examples.bookie.entity.Person;
import lcds.examples.bookie.entity.Subject;

import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;


public class TestBrowsing extends BaseEJBTest {
	
	private IDatabaseConnection connection;
	private IDataSet searchableBooksDataset;

	@BeforeClass
	public void setup() throws Exception {
		connection = getDatabaseConnection();
		URL dsPath = this.getClass().getResource("/lcds/examples/bookie/datasets/searchable_books_dataset.xml");
		searchableBooksDataset = new FlatXmlDataSet(new FileInputStream(dsPath.getFile()));

		DatabaseOperation.CLEAN_INSERT.execute(connection, searchableBooksDataset);
	}
	
	@Test
	public void testSignIn() throws Exception {
		PersonDAO dao = DAOLookupHelper.getInstance().getPersonDAO();
		int user_id = Integer.parseInt((String)searchableBooksDataset.getTable("people").getValue(0, "id"));
		Person user = dao.findById(user_id);
		String cardNumber = (String)searchableBooksDataset.getTable("people").getValue(0, "card_number");
		assertEquals(user.getCardNumber(), cardNumber);
		Person cardHolder = dao.findByCardNumber(cardNumber);
		assertEquals(user, cardHolder);
	}
	
	@Test
	public void testGetAllSubjects() throws Exception {
		SubjectDAO dao = DAOLookupHelper.getInstance().getSubjectDAO();
		List<Subject> subjects = dao.getAll();
		int expectedLength	= searchableBooksDataset.getTable("subjects").getRowCount();
		int actualLength	= subjects.size();
		assertEquals(expectedLength, actualLength);
	}
	
	@Test
	public void testGetAllAuthors() throws Exception {
		PersonDAO dao = DAOLookupHelper.getInstance().getPersonDAO();
		List<Person> authors = dao.getAllAuthors();
		int expectedLength	= searchableBooksDataset.getTable("people").getRowCount() - 1; //subtract one, since we have one test user
		int actualLength	= authors.size();
		assertEquals(expectedLength, actualLength);
	}
	
	@Test
	public void testGetAllBooks() throws Exception {
		BookDAO dao = DAOLookupHelper.getInstance().getBookDAO();
		List<Book> books = dao.getAll();
		int expectedLength	= searchableBooksDataset.getTable("books").getRowCount();
		int actualLength	= books.size();
		assertEquals(expectedLength, actualLength);
	}
	
	@Test
	public void testSimpleSubjectSearch() throws Exception {
		SubjectDAO subjectDAO = DAOLookupHelper.getInstance().getSubjectDAO();
		BookDAO bookDAO = DAOLookupHelper.getInstance().getBookDAO();
		
		int zymurgy_id = Integer.parseInt((String)searchableBooksDataset.getTable("subjects").getValue(3, "id"));
		Subject zymurgy = subjectDAO.findById(zymurgy_id);
		
		Collection<Book> booksOnZymurgy = bookDAO.findBySubject(zymurgy);
		assertEquals(2, booksOnZymurgy.size());
	}
	
	@Test
	public void testSimpleAuthorSearch() throws Exception {
		PersonDAO personDAO = DAOLookupHelper.getInstance().getPersonDAO();
		BookDAO bookDAO = DAOLookupHelper.getInstance().getBookDAO();
		
		int cicero_id = Integer.parseInt((String)searchableBooksDataset.getTable("people").getValue(2, "id"));
		Person cicero = personDAO.findById(cicero_id);
		
		Collection<Book> booksByCicero = bookDAO.findByAuthor(cicero);
		assertEquals(2, booksByCicero.size());
	}
	
	@Test
	public void testSubjectSearchByName() throws Exception {
		String searchString = "physics";
		SubjectDAO dao = DAOLookupHelper.getInstance().getSubjectDAO();
		Collection<Subject> subjectList = dao.findByName(searchString);
		assertEquals(1, subjectList.size());
		Subject astrophysics = dao.findById(1);
		assertTrue(subjectList.contains(astrophysics));
	}
	
	@Test
	public void testAuthorSearchByName() throws Exception {
		String searchString = "C";
		PersonDAO dao = DAOLookupHelper.getInstance().getPersonDAO();
		Collection<Person> authorList = dao.findByName(searchString, true);
		assertEquals(3, authorList.size());
		Person conanDoyle = dao.findById(2);
		Person cicero = dao.findById(3);
		Person charlie = dao.findById(4);
		Person fred = dao.findById(5);
		assertTrue(authorList.contains(conanDoyle));
		assertTrue(authorList.contains(cicero));
		assertTrue(authorList.contains(charlie));
		assertFalse(authorList.contains(fred));
	}
	
	@Test
	public void testBookSearchByTitle() throws Exception {
		String title = "space";
		BookDAO dao = DAOLookupHelper.getInstance().getBookDAO();
		Collection<Book> bookList = dao.findByName(title);
		assertEquals(2, bookList.size());
		Book space = dao.findById(1);
		Book moreSpace = dao.findById(2);
		Book deLegibus = dao.findById(4);
		assertTrue(bookList.contains(space));
		assertTrue(bookList.contains(moreSpace));
		assertFalse(bookList.contains(deLegibus));
	}

	
}
