package lcds.examples.bookie.entity;

import lcds.examples.bookie.BaseEJBTest;
import lcds.examples.bookie.dao.BookDAO;
import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.dao.PersonDAO;
import lcds.examples.bookie.dao.SubjectDAO;
import lcds.examples.bookie.entity.Book;
import lcds.examples.bookie.entity.Person;
import lcds.examples.bookie.entity.Subject;

import org.testng.annotations.Test;


public class TestCrud extends BaseEJBTest {
	
	@Test
	public void testCRUDSubject() throws Exception {
		Subject s = new Subject();
		s.setName("Zymurgy");
		
		SubjectDAO dao = DAOLookupHelper.getInstance().getSubjectDAO();
		s = dao.merge(s);
		Integer id = s.getId();
		assertNotNull(dao.findById(id));
		
		s.setName("Beer Making");
		dao.merge(s);
		assertEquals("Expected Zymurgy to be updated to Beer Making", "Beer Making", dao.findById(id).getName());
		
		dao.remove(s);
		assertNull(dao.findById(id));
	}
	
	@Test
	public void testCRUDAuthor() throws Exception {
		Person author = new Person();
		author.setAuthor(true);
		author.setFirstName("Author");
		author.setLastName("Conan-Doyle");
		
		PersonDAO dao = DAOLookupHelper.getInstance().getPersonDAO();
		author = dao.merge(author);
		Integer id = author.getId();
		assertNotNull(dao.findById(id));
		
		author.setFirstName("Arthur");
		dao.merge(author);
		assertEquals("Expected 'Author' to be updated to 'Arthur'", "Arthur", dao.findById(id).getFirstName());
		
		dao.remove(author);
		assertNull(dao.findById(id));
	}
	
	public void testCRUDBook() throws Exception {
		Book book = new Book();
		book.setIsbn("123456");
		book.setTitle("Books: How to Read Them");
		
		BookDAO dao = DAOLookupHelper.getInstance().getBookDAO();
		book = dao.merge(book);
		Integer id = book.getId();
		assertNotNull(dao.findById(id));
		
		book.setTitle("Books for Dummies");
		dao.merge(book);
		assertEquals("Expected 'Books: How to Read Them' to be updated to 'Books for Dummies'", "Books for Dummies", dao.findById(id).getTitle());
		
		dao.remove(book);
		assertNull(dao.findById(id));
	}
	
}
