package lcds.examples.bookie.assemblers;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import lcds.examples.bookie.dao.BookDAO;
import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.entity.Book;

import flex.data.ChangeObject;
import flex.data.assemblers.AbstractAssembler;

public class BookAssembler extends AbstractAssembler {
	
	@Override
	public Object getItem(Map identity) {
		Integer id = (Integer)identity.get("id");
		try {
			BookDAO dao = getBookDAO();
			return dao.findById(id);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Book", e);
		}
	}

	@Override
	public void createItem(Object newBook) {
		Book book = (Book)newBook;
		try {
			BookDAO dao = getBookDAO();
			dao.merge(book);
		} catch (Exception e) {
			throw new RuntimeException("Unable to save new Book", e);
		}
	}

	@Override
	public void updateItem(Object newVersion, Object oldVersion, List changes) {
		Book book = (Book)newVersion;
		try {
			BookDAO dao = getBookDAO();
			dao.merge(book);
		} catch (Exception e) {
			throw new RuntimeException("Unable to update Book", e);
		}
	}

	@Override
	public void deleteItem(Object bookToDelete) {
		Book book = (Book)bookToDelete;
		try {
			BookDAO dao = getBookDAO();
			dao.removeDetached(book);
		} catch (Exception e) {
			throw new RuntimeException("Unable to delete Book", e);
		}
	}

	public List<ChangeObject> syncBooks(List<ChangeObject> changes) {
		try {
			BookDAO dao = getBookDAO();
			Book book;
			for (ChangeObject change : changes) {
				if (change.isCreate() || change.isUpdate()) {
					book = (Book)change.getNewVersion();
					change.setNewVersion(dao.merge(book));
				} else if (change.isDelete()) {
					book = (Book)change.getPreviousVersion();
					if (book.getId() > 0) {
						dao.removeDetached(book);
					}
				}
			}
			return changes;
		} catch (Exception e) {
			throw new RuntimeException("Unable to sync Book changes", e);
		}
	}

	public Collection getAllBooks() {
		try {
			BookDAO dao = getBookDAO();
			return dao.getAll();
		} catch (Exception e) {
			throw new RuntimeException("Unable to get all Books", e);
		}
	}

	public Collection findByName(String name) {
		try {
			BookDAO dao = getBookDAO();
			return dao.findByName(name);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Book by name", e);
		}
	}

	public boolean shouldAddBookToAllCollection(Object[] params, Object item) {
		return true;
	}
	
	public boolean shouldAddBookToNameCollection(Object[] params, Object item) {
		String name = (String) params[0];
		Book book = (Book) item;
		if (book.getTitle().contains(name)) return true;
		return false;
	}
	
	private BookDAO getBookDAO() throws Exception {
		return DAOLookupHelper.getInstance().getBookDAO();
	}


}
