package lcds.examples.bookie.assemblers;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.dao.PersonDAO;
import lcds.examples.bookie.entity.Person;

import flex.data.ChangeObject;
import flex.data.assemblers.AbstractAssembler;

public class PersonAssembler extends AbstractAssembler {
	
	private static Boolean LIMIT_TO_AUTHORS = true;
	
	@Override
	public Object getItem(Map identity) {
		Integer id = (Integer)identity.get("id");
		try {
			PersonDAO dao = getPersonDAO();
			return dao.findById(id);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Person", e);
		}
	}

	@Override
	public void createItem(Object newPerson) {
		Person person = (Person)newPerson;
		try {
			PersonDAO dao = getPersonDAO();
			dao.merge(person);
		} catch (Exception e) {
			throw new RuntimeException("Unable to save new Person", e);
		}
	}

	@Override
	public void updateItem(Object newVersion, Object oldVersion, List changes) {
		Person person = (Person)newVersion;
		try {
			PersonDAO dao = getPersonDAO();
			dao.merge(person);
		} catch (Exception e) {
			throw new RuntimeException("Unable to update Person", e);
		}
	}

	@Override
	public void deleteItem(Object personToDelete) {
		Person person = (Person)personToDelete;
		try {
			PersonDAO dao = getPersonDAO();
			dao.removeDetached(person);
		} catch (Exception e) {
			throw new RuntimeException("Unable to delete Person", e);
		}
	}

	public List<ChangeObject> syncPeople(List<ChangeObject> changes) {
		try {
			PersonDAO dao = getPersonDAO();
			Person person;
			for (ChangeObject change : changes) {
				if (change.isCreate() || change.isUpdate()) {
					person = (Person)change.getNewVersion();
					change.setNewVersion(dao.merge(person));
				} else if (change.isDelete()) {
					person = (Person)change.getPreviousVersion();
					if (person.getId() > 0) {
						dao.removeDetached(person);
					}
				}
			}
			return changes;
		} catch (Exception e) {
			throw new RuntimeException("Unable to sync Person changes", e);
		}
	}

	public Collection getAllPeople(Boolean authors) {
		try {
			PersonDAO dao = getPersonDAO();
			if (authors == LIMIT_TO_AUTHORS) {
				return dao.getAllAuthors();
			} else {
				return dao.getAll();
			}
		} catch (Exception e) {
			throw new RuntimeException("Unable to get all People", e);
		}
	}

	public Collection findByName(String name, Boolean limitToAuthors) {
		try {
			PersonDAO dao = getPersonDAO();
			return dao.findByName(name, limitToAuthors);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Person by name", e);
		}
	}
	
	public boolean shouldAddPersonToAllCollection(Object[] params, Object item) {
		Boolean limitToAuthors = (Boolean) params[0];
		Person person = (Person) item;
		
		if (limitToAuthors == LIMIT_TO_AUTHORS && !person.isAuthor()) {
			return false;
		}
		
		return true;
	}
	
	public boolean shouldAddPersonToNameCollection(Object[] params, Object item) {
		String name = (String) params[0];
		Boolean limitToAuthors = (Boolean) params[1];
		Person person = (Person) item;
		
		if (limitToAuthors == LIMIT_TO_AUTHORS && !person.isAuthor()) {
			return false;
		}
		
		if (!person.getFirstName().contains(name) && !person.getLastName().contains(name)) {
			return false;
		}
		
		return true;
	}

	private PersonDAO getPersonDAO() throws Exception {
		return DAOLookupHelper.getInstance().getPersonDAO();
	}


}
