package lcds.examples.bookie.assemblers;

import java.util.Collection;
import java.util.List;

import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.dao.ReservationDAO;
import lcds.examples.bookie.entity.Reservation;

import flex.data.ChangeObject;
import flex.data.assemblers.AbstractAssembler;

public class ReservationAssembler extends AbstractAssembler {
	
	private static Boolean CHECKED_OUT = true;
	
	@Override
	public void deleteItem(Object reservationToDelete) {
		Reservation reservation = (Reservation)reservationToDelete;
		try {
			ReservationDAO dao = getReservationDAO();
			dao.removeDetached(reservation);
		} catch (Exception e) {
			throw new RuntimeException("Unable to delete Reservation", e);
		}
	}

	@Override
	public void createItem(Object newReservation) {
		try {
			ReservationDAO dao = getReservationDAO();
			Reservation reservation = (Reservation) newReservation;
			dao.persist(reservation);
		} catch (Exception e) {
			throw new RuntimeException("Unable to create Reservation", e);
		}
	}

	public List<ChangeObject> syncReservations(List<ChangeObject> changes) {
		try {
			ReservationDAO dao = getReservationDAO();
			Reservation reservation;
			for (ChangeObject change : changes) {
				if (change.isCreate() || change.isUpdate()) {
					reservation = (Reservation)change.getNewVersion();
					change.setNewVersion(dao.merge(reservation));
				} else if (change.isDelete()) {
					reservation = (Reservation)change.getPreviousVersion();
					if (reservation.getId() > 0) {
						dao.removeDetached(reservation);
					}
				}
			}
			return changes;
		} catch (Exception e) {
			throw new RuntimeException("Unable to sync Reservation changes", e);
		}
	}
	
	public Collection getAllReservations(Boolean getCheckedOut) {
		try {
			ReservationDAO dao = getReservationDAO();
			if (getCheckedOut == CHECKED_OUT) {
				return dao.getAllCheckedOut();
			} else {
				return dao.getAllAwaitingCheckOut();
			}
		} catch (Exception e) {
			throw new RuntimeException("Unable to get all Reservations", e);
		}
	}
	
	public boolean shouldAddReservationToCollection(Object[] params, Object item) {
		Boolean checkedOut = (Boolean)params[0];
		Reservation reservation = (Reservation) item;
		if (checkedOut == CHECKED_OUT) {
			if (reservation.getReservedOn() == null) {
				return false;
			}
		} else {
			if (reservation.getReservedOn() != null) {
				return false;
			}
		}
		return true;
	}
	
	private ReservationDAO getReservationDAO() throws Exception {
		return DAOLookupHelper.getInstance().getReservationDAO();
	}


}
