package lcds.examples.bookie.assemblers;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import lcds.examples.bookie.dao.DAOLookupHelper;
import lcds.examples.bookie.dao.SubjectDAO;
import lcds.examples.bookie.entity.Subject;

import flex.data.ChangeObject;
import flex.data.assemblers.AbstractAssembler;

public class SubjectAssembler extends AbstractAssembler {
	
	@Override
	public Object getItem(Map identity) {
		Integer id = (Integer)identity.get("id");
		try {
			SubjectDAO dao = getSubjectDAO();
			return dao.findById(id);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Subject", e);
		}
	}

	@Override
	public void createItem(Object newSubject) {
		Subject subject = (Subject)newSubject;
		try {
			SubjectDAO dao = getSubjectDAO();
			dao.merge(subject);
		} catch (Exception e) {
			throw new RuntimeException("Unable to save new Subject", e);
		}
	}

	@Override
	public void updateItem(Object newVersion, Object oldVersion, List changes) {
		Subject subject = (Subject)newVersion;
		try {
			SubjectDAO dao = getSubjectDAO();
			dao.merge(subject);
		} catch (Exception e) {
			throw new RuntimeException("Unable to update Subject", e);
		}
	}

	@Override
	public void deleteItem(Object subjectToDelete) {
		Subject subject = (Subject)subjectToDelete;
		try {
			SubjectDAO dao = getSubjectDAO();
			dao.removeDetached(subject);
		} catch (Exception e) {
			throw new RuntimeException("Unable to delete Subject", e);
		}
	}
	
	public Collection getAllSubjects() {
		try {
			SubjectDAO dao = getSubjectDAO();
			return dao.getAll();
		} catch (Exception e) {
			throw new RuntimeException("Unable to get all Subjects", e);
		}
	}

	public Collection findByName(String name) {
		try {
			SubjectDAO dao = getSubjectDAO();
			return dao.findByName(name);
		} catch (Exception e) {
			throw new RuntimeException("Unable to get Subjects by name", e);
		}
	}
	
	public List<ChangeObject> syncSubjects(List<ChangeObject> changes) {
		try {
			SubjectDAO dao = getSubjectDAO();
			Subject subject;
			for (ChangeObject change : changes) {
				if (change.isCreate() || change.isUpdate()) {
					subject = (Subject)change.getNewVersion();
					change.setNewVersion(dao.merge(subject));
				} else if (change.isDelete()) {
					subject = (Subject)change.getPreviousVersion();
					if (subject.getId() > 0) {
						dao.removeDetached(subject);
					}
				}
			}
			return changes;
		} catch (Exception e) {
			throw new RuntimeException("Unable to sync Subject changes", e);
		}
	}

	public boolean shouldAddSubjectToAllCollection(Object[] params, Object item) {
		return true;
	}
	
	public boolean shouldAddSubjectToNameCollection(Object[] params, Object item) {
		String name = (String) params[0];
		Subject subject = (Subject) item;
		if (subject.getName().contains(name)) return true;
		return false;
	}
	
	private SubjectDAO getSubjectDAO() throws Exception {
		return DAOLookupHelper.getInstance().getSubjectDAO();
	}


}
